#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const os = require("os");
const find_prefix_1 = require("./find-prefix");
const ts_command_line_1 = require("@rushstack/ts-command-line");
const index_1 = require("../index");
const read_config_1 = require("./read-config");
const tsapi_1 = require("@tybys/tsapi");
const command = process.argv[2];
const pkg = require('../../package.json');
if (command === '-v' || command === '--version' || command === '-V') {
    console.log(pkg.version);
    process.exit(0);
}
const targetConfigFile = 'tsgo.config.js';
const commandLineParser = new ts_command_line_1.DynamicCommandLineParser({
    toolFilename: 'tsgo',
    toolDescription: `[v${pkg.version}] ${pkg.description}`
});
class CheckAction extends ts_command_line_1.CommandLineAction {
    constructor() {
        super({
            actionName: 'check',
            summary: 'Type Checking',
            documentation: 'Type Checking'
        });
    }
    onDefineParameters() { }
    async onExecute() {
        var _a;
        const root = (0, find_prefix_1.findPrefix)(process.cwd());
        const config = (0, read_config_1.readConfig)(path.resolve(root, targetConfigFile));
        const result = (0, tsapi_1.typeCheck)(path.resolve(config.root, (_a = config.baseTsconfig) !== null && _a !== void 0 ? _a : 'tsconfig.json'));
        if (result.diagnostics && result.diagnostics.length > 0) {
            throw new Error('Type Check failed');
        }
    }
}
class BuildAction extends ts_command_line_1.CommandLineAction {
    constructor() {
        super({
            actionName: 'build',
            summary: 'Build project',
            documentation: 'Build project'
        });
    }
    onDefineParameters() { }
    async onExecute() {
        const root = (0, find_prefix_1.findPrefix)(process.cwd());
        await (0, index_1.build)((0, read_config_1.readConfig)(path.resolve(root, targetConfigFile)));
    }
}
class WatchAction extends ts_command_line_1.CommandLineAction {
    constructor() {
        super({
            actionName: 'watch',
            summary: 'Develop with watch mode',
            documentation: 'Develop with watch mode'
        });
    }
    onDefineParameters() { }
    async onExecute() {
        const root = (0, find_prefix_1.findPrefix)(process.cwd());
        (0, index_1.watch)((0, read_config_1.readConfig)(path.resolve(root, targetConfigFile)));
    }
}
class InitAction extends ts_command_line_1.CommandLineAction {
    constructor() {
        super({
            actionName: 'init',
            summary: 'Create a new library project',
            documentation: 'Create a new library project'
        });
    }
    onDefineParameters() {
        this.defineCommandLineRemainder({
            description: 'package name'
        });
    }
    _copyTemplate(dest, data) {
        const templateDir = path.join(__dirname, '../../template');
        this._copyItem(templateDir, dest, data);
    }
    _copyItem(src, dest, data) {
        const info = fs.statSync(src);
        const rewriteDest = path.join(path.dirname(dest), path.basename(dest).replace(/^__/, '').replace(/^_/, '.'));
        if (info.isDirectory()) {
            fs.mkdirSync(rewriteDest, { recursive: true });
            const items = fs.readdirSync(src);
            for (let i = 0; i < items.length; ++i) {
                this._copyItem(path.join(src, items[i]), path.join(rewriteDest, items[i]), data);
            }
        }
        else if (info.isFile()) {
            const keys = Object.keys(data);
            let content = fs.readFileSync(src, 'utf8');
            for (let i = 0; i < keys.length; ++i) {
                content = content.replace(new RegExp(`\\$\\{${keys[i]}\\}`, 'g'), data[keys[i]]);
            }
            fs.writeFileSync(rewriteDest, content, 'utf8');
        }
    }
    async onExecute() {
        const packageName = this.remainder.values[0];
        if (!packageName) {
            throw new Error('Package name is required');
        }
        const unscopedPackageName = path.posix.basename(packageName);
        const root = path.resolve(process.cwd(), unscopedPackageName);
        if (fs.existsSync(root)) {
            throw new Error(`Can not mkdir "${root}"`);
        }
        fs.mkdirSync(root, { recursive: true });
        this._copyTemplate(root, {
            packageName,
            unscopedPackageName,
            tsgoVersion: pkg.version,
            username: os.userInfo().username
        });
    }
}
commandLineParser.addAction(new InitAction());
commandLineParser.addAction(new CheckAction());
commandLineParser.addAction(new BuildAction());
commandLineParser.addAction(new WatchAction());
commandLineParser.executeWithoutErrorHandling().catch(err => {
    console.error(err);
    process.exit(1);
});
